@@ -18,15 +18,16 @@ AVFormatContext *ocaml_av_get_format_context(value *p_av);
 #define InputFormat_val(v)                                                     \
   (*(avioformat_const AVInputFormat **)Data_abstract_val(v))
 
-void value_of_inputFormat(avioformat_const AVInputFormat *inputFormat,
-                          value *p_value);
+void value_of_inputFormat(value *p_value,
+                          avioformat_const AVInputFormat *inputFormat);
 
 /***** AVOutputFormat *****/
 
 #define OutputFormat_val(v)                                                    \
   (*(avioformat_const AVOutputFormat **)Data_abstract_val(v))
 
-value value_of_outputFormat(avioformat_const AVOutputFormat *outputFormat);
+void value_of_outputFormat(value *p_value,
+                           avioformat_const AVOutputFormat *outputFormat);
 
 /***** Control message *****/
 value *ocaml_av_get_control_message_callback(struct AVFormatContext *ctx);
