@@ -35,10 +35,6 @@ CAMLprim value ocaml_avcodec_flag_qscale(value unit) {
   return Val_int(AV_CODEC_FLAG_QSCALE);
 }
 
-CAMLprim value ocaml_avcodec_get_input_buffer_padding_size(value unit) {
-  return Val_int(AV_INPUT_BUFFER_PADDING_SIZE);
-}
-
 CAMLprim value ocaml_avcodec_subtitle_codec_id_to_AVCodecID(value _codec_id) {
   return Val_int(SubtitleCodecID_val(_codec_id));
 }
@@ -52,9 +48,8 @@ static AVCodecContext *create_AVCodecContext(AVCodecParameters *params,
 
   codec_context = avcodec_alloc_context3(codec);
 
-  if (!codec_context) {
+  if (!codec_context)
     caml_raise_out_of_memory();
-  }
 
   if (params)
     ret = avcodec_parameters_to_context(codec_context, params);
@@ -100,8 +95,10 @@ void value_of_codec_parameters_copy(AVCodecParameters *src, value *pvalue) {
 
   int ret = avcodec_parameters_copy(dst, src);
 
-  if (ret < 0)
+  if (ret < 0) {
+    avcodec_parameters_free(&dst);
     ocaml_avutil_raise_error(ret);
+  }
 
   *pvalue = caml_alloc_custom(&codec_parameters_ops,
                               sizeof(AVCodecParameters *), 0, 1);
@@ -119,9 +116,7 @@ static struct custom_operations packet_ops = {
     "ocaml_packet",      finalize_packet,          custom_compare_default,
     custom_hash_default, custom_serialize_default, custom_deserialize_default};
 
-value value_of_ffmpeg_packet(AVPacket *packet) {
-  value ret;
-
+value value_of_ffmpeg_packet(value *ret, AVPacket *packet) {
   if (!packet)
     Fail("Empty packet");
 
@@ -130,14 +125,14 @@ value value_of_ffmpeg_packet(AVPacket *packet) {
   if (packet->buf)
     size = packet->buf->size;
 
-  ret = caml_alloc_custom_mem(&packet_ops, sizeof(AVPacket *), size);
-  Packet_val(ret) = packet;
-
-  return ret;
+  *ret = caml_alloc_custom_mem(&packet_ops, sizeof(AVPacket *), size);
+  Packet_val(*ret) = packet;
+  return *ret;
 }
 
 CAMLprim value ocaml_avcodec_create_packet(value _data) {
   CAMLparam1(_data);
+  CAMLlocal1(ret);
   AVPacket *packet;
   int len = caml_string_length(_data);
 
@@ -153,7 +148,7 @@ CAMLprim value ocaml_avcodec_create_packet(value _data) {
 
   memcpy(packet->data, String_val(_data), len);
 
-  CAMLreturn(value_of_ffmpeg_packet(packet));
+  CAMLreturn(value_of_ffmpeg_packet(&ret, packet));
 }
 
 CAMLprim value ocaml_avcodec_packet_content(value _packet) {
@@ -437,121 +432,6 @@ CAMLprim value ocaml_avcodec_packet_to_bytes(value _packet) {
   CAMLreturn(ans);
 }
 
-/***** AVCodecParserContext *****/
-
-typedef struct {
-  AVCodecParserContext *context;
-  AVCodecContext *codec_context;
-} parser_t;
-
-#define Parser_val(v) (*(parser_t **)Data_custom_val(v))
-
-static void free_parser(parser_t *parser) {
-  if (!parser)
-    return;
-
-  if (parser->context) {
-    caml_release_runtime_system();
-    av_parser_close(parser->context);
-    caml_acquire_runtime_system();
-  }
-
-  if (parser->codec_context)
-    avcodec_free_context(&parser->codec_context);
-
-  av_free(parser);
-}
-
-static void finalize_parser(value v) { free_parser(Parser_val(v)); }
-
-static struct custom_operations parser_ops = {
-    "ocaml_avcodec_parser",   finalize_parser,
-    custom_compare_default,   custom_hash_default,
-    custom_serialize_default, custom_deserialize_default};
-
-static parser_t *create_parser(AVCodecParameters *params,
-                               const AVCodec *codec) {
-  parser_t *parser = (parser_t *)av_mallocz(sizeof(parser_t));
-  if (!parser)
-    caml_raise_out_of_memory();
-
-  caml_release_runtime_system();
-  parser->context = av_parser_init(codec->id);
-  caml_acquire_runtime_system();
-
-  if (!parser->context) {
-    free_parser(parser);
-    caml_raise_out_of_memory();
-  }
-
-  parser->codec_context = create_AVCodecContext(NULL, codec);
-
-  return parser;
-}
-
-CAMLprim value ocaml_avcodec_create_parser(value _params, value _codec) {
-  CAMLparam2(_params, _codec);
-  CAMLlocal1(ans);
-  const AVCodec *codec = AvCodec_val(_codec);
-  AVCodecParameters *params = NULL;
-
-  if (_params != Val_none)
-    params = CodecParameters_val(Field(_params, 0));
-
-  parser_t *parser = create_parser(params, codec);
-
-  ans = caml_alloc_custom(&parser_ops, sizeof(parser_t *), 0, 1);
-  Parser_val(ans) = parser;
-
-  CAMLreturn(ans);
-}
-
-CAMLprim value ocaml_avcodec_parse_packet(value _parser, value _data,
-                                          value _ofs, value _len) {
-  CAMLparam2(_parser, _data);
-  CAMLlocal3(val_packet, tuple, ans);
-  parser_t *parser = Parser_val(_parser);
-  uint8_t *data = Caml_ba_data_val(_data) + Int_val(_ofs);
-  size_t init_len = Int_val(_len);
-  size_t len = init_len;
-  int ret = 0;
-
-  AVPacket *packet = av_packet_alloc();
-
-  if (!packet)
-    caml_raise_out_of_memory();
-
-  do {
-    ret = av_parser_parse2(parser->context, parser->codec_context,
-                           &packet->data, &packet->size, data, len,
-                           AV_NOPTS_VALUE, AV_NOPTS_VALUE, 0);
-    data += ret;
-    len -= ret;
-  } while (packet->size == 0 && ret > 0);
-
-  if (ret < 0) {
-    av_packet_free(&packet);
-    ocaml_avutil_raise_error(ret);
-  }
-
-  if (packet->size) {
-    val_packet = value_of_ffmpeg_packet(packet);
-
-    tuple = caml_alloc_tuple(2);
-
-    Store_field(tuple, 0, val_packet);
-    Store_field(tuple, 1, Val_int(init_len - len));
-
-    ans = caml_alloc(1, 0);
-    Store_field(ans, 0, tuple);
-  } else {
-    av_packet_free(&packet);
-    ans = Val_int(0);
-  }
-
-  CAMLreturn(ans);
-}
-
 /***** codec_context_t *****/
 
 typedef struct {
@@ -663,8 +543,10 @@ CAMLprim value ocaml_avcodec_create_audio_encoder(value _sample_fmt,
   }
 
   codec_context_t *ctx = (codec_context_t *)av_mallocz(sizeof(codec_context_t));
-  if (!ctx)
+  if (!ctx) {
+    av_dict_free(&options);
     caml_raise_out_of_memory();
+  }
 
   ans = caml_alloc_custom(&codec_context_ops, sizeof(codec_context_t *), 0, 1);
   CodecContext_val(ans) = ctx;
@@ -674,6 +556,7 @@ CAMLprim value ocaml_avcodec_create_audio_encoder(value _sample_fmt,
   ctx->codec_context = avcodec_alloc_context3(codec);
 
   if (!ctx->codec_context) {
+    av_dict_free(&options);
     caml_raise_out_of_memory();
   }
 
@@ -681,16 +564,20 @@ CAMLprim value ocaml_avcodec_create_audio_encoder(value _sample_fmt,
 
   err = av_channel_layout_copy(&ctx->codec_context->ch_layout,
                                AVChannelLayout_val(_channel_layout));
-  if (err < 0)
+  if (err < 0) {
+    av_dict_free(&options);
     ocaml_avutil_raise_error(err);
+  }
 
   // Open the codec
   caml_release_runtime_system();
   err = avcodec_open2(ctx->codec_context, ctx->codec, &options);
   caml_acquire_runtime_system();
 
-  if (err < 0)
+  if (err < 0) {
+    av_dict_free(&options);
     ocaml_avutil_raise_error(err);
+  }
 
   // Return unused keys
   count = av_dict_count(options);
@@ -837,43 +724,13 @@ CAMLprim value ocaml_avcodec_receive_frame(value _ctx) {
   int ret = 0;
 
   AVFrame *frame = av_frame_alloc();
+  AVFrame *hw_frame = NULL;
+  AVFrame *effective_frame = frame;
 
   if (!frame) {
     caml_raise_out_of_memory();
   }
 
-  if (ctx->codec_context->hw_frames_ctx && frame) {
-    AVFrame *hw_frame = av_frame_alloc();
-
-    if (!hw_frame) {
-      caml_raise_out_of_memory();
-    }
-
-    caml_release_runtime_system();
-    ret = av_hwframe_get_buffer(ctx->codec_context->hw_frames_ctx, hw_frame, 0);
-    caml_acquire_runtime_system();
-
-    if (ret < 0) {
-      av_frame_free(&hw_frame);
-      ocaml_avutil_raise_error(ret);
-    }
-
-    if (!hw_frame->hw_frames_ctx) {
-      caml_raise_out_of_memory();
-    }
-
-    caml_release_runtime_system();
-    ret = av_hwframe_transfer_data(hw_frame, frame, 0);
-    caml_acquire_runtime_system();
-
-    if (ret < 0) {
-      av_frame_free(&hw_frame);
-      ocaml_avutil_raise_error(ret);
-    }
-
-    frame = hw_frame;
-  }
-
   caml_release_runtime_system();
   ret = avcodec_receive_frame(ctx->codec_context, frame);
   caml_acquire_runtime_system();
@@ -884,13 +741,32 @@ CAMLprim value ocaml_avcodec_receive_frame(value _ctx) {
   }
 
   if (ret == AVERROR(EAGAIN)) {
-    ans = Val_int(0);
-  } else {
-    ans = caml_alloc(1, 0);
-    value_of_frame(&val_frame, frame);
-    Store_field(ans, 0, val_frame);
+    av_frame_free(&frame);
+    CAMLreturn(Val_none);
   }
 
+  if (ctx->codec_context->hw_frames_ctx) {
+    hw_frame = av_frame_alloc();
+    if (!hw_frame) {
+      av_frame_free(&frame);
+      caml_raise_out_of_memory();
+    }
+
+    ret = av_hwframe_transfer_data(hw_frame, frame, 0);
+    if (ret < 0) {
+      av_frame_free(&frame);
+      av_frame_free(&hw_frame);
+      ocaml_avutil_raise_error(ret);
+    }
+
+    av_frame_free(&frame);
+    effective_frame = hw_frame;
+  }
+
+  ans = caml_alloc_tuple(1);
+  value_of_frame(&val_frame, effective_frame);
+  Store_field(ans, 0, val_frame);
+
   CAMLreturn(ans);
 }
 
@@ -963,7 +839,6 @@ CAMLprim value ocaml_avcodec_receive_packet(value _ctx) {
   CAMLlocal2(val_packet, ans);
   codec_context_t *ctx = CodecContext_val(_ctx);
   int ret = 0;
-
   AVPacket *packet = av_packet_alloc();
 
   if (!packet)
@@ -977,14 +852,15 @@ CAMLprim value ocaml_avcodec_receive_packet(value _ctx) {
     av_packet_free(&packet);
 
     if (ret == AVERROR(EAGAIN) || ret == AVERROR_EOF)
-      ans = Val_int(0);
-    else
-      ocaml_avutil_raise_error(ret);
-  } else {
-    ans = caml_alloc(1, 0);
-    val_packet = value_of_ffmpeg_packet(packet);
-    Store_field(ans, 0, val_packet);
+      CAMLreturn(Val_none);
+
+    ocaml_avutil_raise_error(ret);
   }
+
+  ans = caml_alloc_tuple(1);
+  val_packet = value_of_ffmpeg_packet(&val_packet, packet);
+  Store_field(ans, 0, val_packet);
+
   CAMLreturn(ans);
 }
 
@@ -1235,7 +1111,7 @@ CAMLprim value ocaml_avcodec_hw_methods(value _codec) {
 
   cons1 = Val_int(0);
   do {
-    ret = caml_alloc(2, 0);
+    ret = caml_alloc_tuple(2);
     Store_field(ret, 1, cons1);
 
     tmp1 = caml_alloc_tuple(3);
@@ -1245,7 +1121,7 @@ CAMLprim value ocaml_avcodec_hw_methods(value _codec) {
     cons2 = Val_int(0);
     for (n = 0; n < AV_CODEC_HW_CONFIG_METHOD_T_TAB_LEN; n++) {
       if (config->methods & AV_CODEC_HW_CONFIG_METHOD_T_TAB[n][1]) {
-        tmp2 = caml_alloc(2, 0);
+        tmp2 = caml_alloc_tuple(2);
         Store_field(tmp2, 0, AV_CODEC_HW_CONFIG_METHOD_T_TAB[n][0]);
         Store_field(tmp2, 1, cons2);
         cons2 = tmp2;
@@ -1270,10 +1146,24 @@ CAMLprim value ocaml_avcodec_get_supported_channel_layouts(value _codec) {
   int i;
   List_init(list);
   const AVCodec *codec = AvCodec_val(_codec);
+  const AVChannelLayout *ch_layouts = NULL;
 
-  if (codec->ch_layouts) {
-    for (i = 0; codec->ch_layouts[i].nb_channels != 0; i++) {
-      value_of_channel_layout(&ch_layout, &codec->ch_layouts[i]);
+#if LIBAVCODEC_VERSION_INT <= AV_VERSION_INT(61, 13, 100)
+  ch_layouts = codec->ch_layouts;
+#else
+  int err;
+
+  err =
+      avcodec_get_supported_config(NULL, codec, AV_CODEC_CONFIG_CHANNEL_LAYOUT,
+                                   0, (const void **)&ch_layouts, NULL);
+
+  if (err < 0)
+    ocaml_avutil_raise_error(err);
+#endif
+
+  if (ch_layouts) {
+    for (i = 0; ch_layouts[i].nb_channels != 0; i++) {
+      value_of_channel_layout(&ch_layout, &ch_layouts[i]);
       List_add(list, cons, ch_layout);
     }
   }
@@ -1287,10 +1177,23 @@ CAMLprim value ocaml_avcodec_get_supported_sample_formats(value _codec) {
   int i;
   List_init(list);
   const AVCodec *codec = AvCodec_val(_codec);
+  const enum AVSampleFormat *sample_fmts = NULL;
 
-  if (codec->sample_fmts) {
-    for (i = 0; codec->sample_fmts[i] != -1; i++)
-      List_add(list, cons, Val_SampleFormat(codec->sample_fmts[i]));
+#if LIBAVCODEC_VERSION_INT <= AV_VERSION_INT(61, 13, 100)
+  sample_fmts = codec->sample_fmts;
+#else
+  int err;
+
+  err = avcodec_get_supported_config(NULL, codec, AV_CODEC_CONFIG_SAMPLE_FORMAT,
+                                     0, (const void **)&sample_fmts, NULL);
+
+  if (err < 0)
+    ocaml_avutil_raise_error(err);
+#endif
+
+  if (sample_fmts) {
+    for (i = 0; sample_fmts[i] != -1; i++)
+      List_add(list, cons, Val_SampleFormat(sample_fmts[i]));
   }
 
   CAMLreturn(list);
@@ -1302,10 +1205,24 @@ CAMLprim value ocaml_avcodec_get_supported_sample_rates(value _codec) {
   int i;
   List_init(list);
   const AVCodec *codec = AvCodec_val(_codec);
+  const int *supported_samplerates = NULL;
 
-  if (codec->supported_samplerates) {
-    for (i = 0; codec->supported_samplerates[i] != 0; i++)
-      List_add(list, cons, Val_int(codec->supported_samplerates[i]));
+#if LIBAVCODEC_VERSION_INT <= AV_VERSION_INT(61, 13, 100)
+  supported_samplerates = codec->supported_samplerates;
+#else
+  int err;
+
+  err =
+      avcodec_get_supported_config(NULL, codec, AV_CODEC_CONFIG_SAMPLE_RATE, 0,
+                                   (const void **)&supported_samplerates, NULL);
+
+  if (err < 0)
+    ocaml_avutil_raise_error(err);
+#endif
+
+  if (supported_samplerates) {
+    for (i = 0; supported_samplerates[i] != 0; i++)
+      List_add(list, cons, Val_int(supported_samplerates[i]));
   }
 
   CAMLreturn(list);
@@ -1384,10 +1301,24 @@ CAMLprim value ocaml_avcodec_get_supported_frame_rates(value _codec) {
   int i;
   List_init(list);
   const AVCodec *codec = AvCodec_val(_codec);
+  const AVRational *supported_framerates = NULL;
 
-  if (codec->supported_framerates) {
-    for (i = 0; codec->supported_framerates[i].num != 0; i++) {
-      value_of_rational(&codec->supported_framerates[i], &val);
+#if LIBAVCODEC_VERSION_INT <= AV_VERSION_INT(61, 13, 100)
+  supported_framerates = codec->supported_framerates;
+#else
+  int err;
+
+  err =
+      avcodec_get_supported_config(NULL, codec, AV_CODEC_CONFIG_FRAME_RATE, 0,
+                                   (const void **)&supported_framerates, NULL);
+
+  if (err < 0)
+    ocaml_avutil_raise_error(err);
+#endif
+
+  if (supported_framerates) {
+    for (i = 0; supported_framerates[i].num != 0; i++) {
+      value_of_rational(&supported_framerates[i], &val);
       List_add(list, cons, val);
     }
   }
@@ -1401,10 +1332,23 @@ CAMLprim value ocaml_avcodec_get_supported_pixel_formats(value _codec) {
   int i;
   List_init(list);
   const AVCodec *codec = AvCodec_val(_codec);
+  const enum AVPixelFormat *pix_fmts = NULL;
 
-  if (codec->pix_fmts) {
-    for (i = 0; codec->pix_fmts[i] != -1; i++)
-      List_add(list, cons, Val_PixelFormat(codec->pix_fmts[i]));
+#if LIBAVCODEC_VERSION_INT <= AV_VERSION_INT(61, 13, 100)
+  pix_fmts = codec->pix_fmts;
+#else
+  int err;
+
+  err = avcodec_get_supported_config(NULL, codec, AV_CODEC_CONFIG_PIX_FORMAT, 0,
+                                     (const void **)&pix_fmts, NULL);
+
+  if (err < 0)
+    ocaml_avutil_raise_error(err);
+#endif
+
+  if (pix_fmts) {
+    for (i = 0; pix_fmts[i] != -1; i++)
+      List_add(list, cons, Val_PixelFormat(pix_fmts[i]));
   }
 
   CAMLreturn(list);
@@ -1829,6 +1773,7 @@ CAMLprim value ocaml_avcodec_bsf_send_eof(value _filter) {
 
 CAMLprim value ocaml_avcodec_bsf_receive_packet(value _filter) {
   CAMLparam1(_filter);
+  CAMLlocal1(ans);
   int ret;
   AVPacket *packet;
 
@@ -1847,5 +1792,5 @@ CAMLprim value ocaml_avcodec_bsf_receive_packet(value _filter) {
     ocaml_avutil_raise_error(ret);
   }
 
-  CAMLreturn(value_of_ffmpeg_packet(packet));
+  CAMLreturn(value_of_ffmpeg_packet(&ans, packet));
 }
